@@ -9,16 +9,17 @@ import datetime
 from os.path import join, split, exists
 from d3m_interface.basic_ta3 import BasicTA3
 from d3m_interface.visualization import plot_metadata, plot_comparison_pipelines, plot_text_summary
-from d3m_interface.data_converter import is_d3m_format, convert_d3m_format, convert_d3mtext_dataframe, copy_folder
+from d3m_interface.data_converter import is_d3m_format, dataset_to_d3m, d3mtext_to_dataframe, copy_folder, to_d3m_json
 from d3m.metadata.problem import PerformanceMetric
 from threading import Thread
+from IPython.core.getipython import get_ipython
 
 
 logging.basicConfig(level=logging.INFO, format='%(levelname)s: %(message)s', stream=sys.stdout)
 logger = logging.getLogger(__name__)
 pd.set_option('display.max_colwidth', None)
 
-TA2_DOCKER_IMAGES = {'NYU': 'registry.gitlab.com/vida-nyu/d3m/ta2:latest',
+TA2_DOCKER_IMAGES = {'AlphaD3M': 'registry.gitlab.com/vida-nyu/d3m/ta2:latest',
                      'CMU': 'registry.datadrivendiscovery.org/sheath/cmu-ta2:latest',
                      'SRI': 'registry.gitlab.com/daraghhartnett/autoflow:latest',
                      'TAMU': 'dmartinez05/tamuta2:latest'}
@@ -40,12 +41,12 @@ def kernel_interrupt_handler(signal, frame):
 
 class Automl:
 
-    def __init__(self, output_folder, ta2_id='NYU'):
+    def __init__(self, output_folder, ta2_id='AlphaD3M'):
         """Create/instantiate an Automl object
 
         :param output_folder: Path to the output directory
         :param ta2_id: TA2 system name. It makes reference to the TA2 docker image. The provided TA2 systems are the
-            following: `NYU, CMU, SRI, TAMU`
+            following: `AlphaD3M, CMU, SRI, TAMU`
         """
         if ta2_id not in TA2_DOCKER_IMAGES:
             raise ValueError('Unknown "%s" TA2, you should choose among: [%s]' % (ta2_id, ', '.join(TA2_DOCKER_IMAGES)))
@@ -92,7 +93,7 @@ class Automl:
         if not is_d3m_format(dataset, suffix):
             self.problem_config = {'target_column': target, 'metric': metric, 'task_keywords': task_keywords,
                                    'optional': kwargs}
-            dataset = convert_d3m_format(dataset, self.output_folder, self.problem_config, suffix)
+            dataset = dataset_to_d3m(dataset, self.output_folder, self.problem_config, suffix)
 
         self.dataset = split(dataset)[0]
         self.start_ta2()
@@ -206,7 +207,7 @@ class Automl:
         dataset_in_container = '/input/dataset/TEST/dataset_TEST/datasetDoc.json'
 
         if not is_d3m_format(test_dataset, suffix):
-            convert_d3m_format(test_dataset, self.output_folder, self.problem_config, suffix)
+            dataset_to_d3m(test_dataset, self.output_folder, self.problem_config, suffix)
         elif test_dataset != join(self.dataset, 'TEST'):  # Special case for D3M test dataset with different path
             destination_path = join(self.output_folder, 'temp', 'dataset_d3mformat', 'TEST')
             copy_folder(test_dataset, destination_path)
@@ -249,20 +250,25 @@ class Automl:
     def score(self, pipeline_id, test_dataset):
         """Compute a proper score of the model
 
-        :param pipeline_id: Pipeline id
+        :param pipeline_id: A id of a pipeline or a Pipeline object
         :param test_dataset: Path to dataset. It supports D3M dataset, and CSV file
         :returns: A tuple holding metric name and score value
         """
         suffix = 'SCORE'
 
-        if pipeline_id not in self.pipelines:
-            raise ValueError('Pipeline id=%s does not exist' % pipeline_id)
-
         if not is_d3m_format(test_dataset, suffix):
-            convert_d3m_format(test_dataset, self.output_folder, self.problem_config, suffix)
+            dataset_to_d3m(test_dataset, self.output_folder, self.problem_config, suffix)
+
+        if not isinstance(pipeline_id, str):
+            d3m_json = to_d3m_json(pipeline_id)
+            pipeline_id = d3m_json['id']
+        else:
+            if pipeline_id not in self.pipelines:
+                raise ValueError('Pipeline id=%s does not exist' % pipeline_id)
+            d3m_json = self.pipelines[pipeline_id]['json_representation']
 
         with open(join(self.output_folder, '%s.json' % pipeline_id), 'w') as fout:
-            json.dump(self.pipelines[pipeline_id]['json_representation'], fout)  # Save temporally the json pipeline
+            json.dump(d3m_json, fout)  # Save temporally the json pipeline
 
         dataset_in_container = '/input/dataset/'
         dataset_train_path = join(dataset_in_container, 'TRAIN/dataset_TRAIN/datasetDoc.json')
@@ -362,6 +368,73 @@ class Automl:
 
         return profiler_inputs
 
+    def export_pipeline_code(self, pipeline_id, ipython_cell=True):
+        """Converts a Pipeline Description to an executable python script
+
+        :param pipeline_id: Pipeline id
+        :param ipython_cell: Whether or not to show the Python code in a Jupyter Notebook cell
+        """
+        pipeline_template = self.pipelines[pipeline_id]['json_representation']
+        code = "from d3m_interface.pipeline import Pipeline\n\n"
+        code += "pipeline = Pipeline()\n\n"
+        code += "input_data = pipeline.make_pipeline_input()\n"
+
+        prev_step = None
+        prev_steps = {}
+        count_template_steps = 0
+        for pipeline_step in pipeline_template['steps']:
+            if pipeline_step['type'] == 'PRIMITIVE':
+                code += f"""\nstep_{count_template_steps} = pipeline.make_pipeline_step('{pipeline_step['primitive'][
+                    'python_path']}')\n"""
+                if 'outputs' in pipeline_step:
+                    for output in pipeline_step['outputs']:
+                        prev_steps['steps.%d.%s' % (count_template_steps, output['id'])] = "step_%d" % (
+                            count_template_steps)
+
+                if 'hyperparams' in pipeline_step:
+                    code += f"""pipeline.set_hyperparams(step_{count_template_steps}"""
+                    for hyper, desc in pipeline_step['hyperparams'].items():
+                        if desc['type'] == 'VALUE':
+                            code += f""", {hyper}={"'" + desc['data'] + "'" if type(desc['data']) == str else desc[
+                                'data']}"""
+                        else:
+                            code += f""", {hyper}= {"{"}'type':'{desc['type']}' ,'data':{"'" + desc[
+                                'data'] + "'" if type(desc['data']) == str else desc['data']}{"}"}"""
+                    code += f""")\n"""
+
+            else:
+                # TODO In the future we should be able to handle subpipelines
+                break
+            if prev_step:
+                if 'arguments' in pipeline_step:
+                    for argument, desc in pipeline_step['arguments'].items():
+                        from_output = desc['data'].split('.')[-1]
+                        to_input = argument
+                        code += f"""pipeline.connect({prev_steps[desc['data']]}, step_{count_template_steps}"""
+
+                        if from_output != 'produce':
+                            code += f""", from_output='{from_output}'"""
+
+                        if to_input != 'inputs':
+                            code += f""", to_input='{argument}'"""
+                        code += f""")\n"""
+            else:
+                code += f"""pipeline.connect(input_data, step_{count_template_steps}, from_output='dataset')\n"""
+            prev_step = "step_%d" % count_template_steps
+            count_template_steps += 1
+
+        if ipython_cell:
+            shell = get_ipython()
+
+            payload = dict(
+                source='set_next_input',
+                text=code,
+                replace=False,
+            )
+            shell.payload_manager.write_payload(payload, single=False)
+        else:
+            return code
+
     def end_session(self):
         """This safely ends session in D3M interface
         """
@@ -372,7 +445,7 @@ class Automl:
         logger.info('Session ended!')
 
     def start_ta2(self):
-        logger.info('Initializing %s TA2...', self.ta2_id)
+        logger.info('Initializing %s AutoML...', self.ta2_id)
         subprocess.call(['docker', 'stop', 'ta2_container'])
 
         self.ta2 = subprocess.Popen(
@@ -394,7 +467,7 @@ class Automl:
             try:
                 self.ta3 = BasicTA3()
                 self.ta3.do_hello()
-                logger.info('%s TA2 initialized!', self.ta2_id)
+                logger.info('%s AutoML initialized!', self.ta2_id)
                 break
             except:
                 if self.ta3.channel is not None:
@@ -451,15 +524,15 @@ class Automl:
         pipelineprofiler_inputs = self.create_pipelineprofiler_inputs(test_dataset, source_name)
         plot_comparison_pipelines(pipelineprofiler_inputs)
 
-    def plot_text_analysis(self, dataset_path, category_column, text_column):
+    def plot_text_analysis(self, dataset_path, label_column, text_column):
         """ Plot a visualization for text datasets
 
         :param dataset_path: Path to dataset.  It supports D3M dataset
-        :param category_column: Name of the column that contains the categories
+        :param label_column: Name of the column that contains the categories
         :param text_column: Name of the column that contains the texts
         """
-        dataframe = convert_d3mtext_dataframe(dataset_path, text_column)
-        plot_text_summary(dataframe, category_column, text_column)
+        dataframe = d3mtext_to_dataframe(dataset_path, text_column)
+        plot_text_summary(dataframe, text_column, label_column)
 
     @staticmethod
     def add_new_ta2(ta2_id, docker_image_url):
